import React from "react";
import Prism from "prismjs";
import createPrismPlugin from "draft-js-prism-plugin";
import { EditorState } from "draft-js";
import { getDefaultKeyBinding } from "draft-js";

const CODE_COMMAND = "code-highlight";
const prismPlugin = createPrismPlugin({
  // It's required to provide your own instance of Prism
  prism: Prism
});

function codeHighlightingEditor(Editor) {
  return class extends React.Component {
    constructor(props) {
      super(props);
      this.handleKeyCommand = this.handleKeyCommand.bind(this);
      this.keyBinding = this.keyBinding.bind(this);
    }

    componentDidUpdate(prevProps) {
      const simulatedKeyCommand =
        prevProps.simulatedKeyCommand === null &&
        this.props.simulatedKeyCommand !== null;
      if (simulatedKeyCommand) {
        this.handleKeyCommand(
          this.props.simulatedKeyCommand,
          this.props.ourEditorState
        );
      }
    }

    handleKeyCommand(command, editorState) {
      if (command === CODE_COMMAND) {
        const key = editorState.getSelection().getAnchorKey();
        const currentContent = editorState.getCurrentContent();
        const blockMap = currentContent.getBlockMap();
        const block = currentContent.getBlockForKey(key);

        const data = block.getData().merge({ language: "javascript" });
        const newBlock = block.merge({ data });
        const newContentState = currentContent.merge({
          blockMap: blockMap.set(key, newBlock),
          selectionAfter: currentSelection
        });
        const newEditorState = EditorState.push(
          editorState,
          newContentState,
          "change-block-data"
        );

        this.props.onChange(newEditorState);
        return "handled";
      }
      if (this.props.handleKeyCommand) {
        return this.props.handleKeyCommand(command, editorState);
      }
      return "not-handled";
    }

    keyBinding(e) {
      if (e.keyCode === 78 && hasCommandModifier(e)) {
        //command+n
        return CODE_COMMAND;
      }
      if (this.props.keyBindingFn) {
        return this.props.keyBindingFn(e);
      }
      return getDefaultKeyBinding(e);
    }

    render() {
      let plugins = [];
      if (this.props.plugins) {
        plugins = this.props.plugins;
      }
      plugins.push(prismPlugin);

      const {
        handleKeyCommand,
        keyBindingFn,
        simulatedKeyCommand,
        ...props
      } = this.props;
      return (
        <Editor
          plugins={plugins}
          handleKeyCommand={this.handleKeyCommand}
          keyBindingFn={this.keyBinding}
          {...props}
        />
      );
    }
  };
}

window.toolbar.buttons.push({
  icon: "💻",
  command: CODE_COMMAND,
  hint: "Apply code highlighting to selected text"
});

module.exports.mutations = {
  BaseEditor: codeHighlightingEditor
};
